#include <Wire.h>

#define FREF 8000000
#define FVCO1_START 2000000000
#define FVCO1_STOP  2500000000
#define FVCO2_START 2500000000
#define FVCO2_STOP  3000000000
#define FVCO3_START 3000000000
#define FVCO3_STOP  3500000000


void TWIStart(void)
{
    TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
    while ((TWCR & (1<<TWINT)) == 0);
}
void TWIStop(void)
{
    TWCR = (1<<TWINT)|(1<<TWSTO)|(1<<TWEN);
    while(TWCR&(1<<TWSTO));  
}
void TWIWrite(uint8_t u8data)
{
    TWDR = u8data;
    TWCR = (1<<TWINT)|(1<<TWEN);
    while ((TWCR & (1<<TWINT)) == 0);
}


void setup()
{
  //i2c init
  TWBR = 0xff;  //scl about 30kHz, no prescaler (TWSR left 0x00)

  //init spi
  DDRB |= (1<<5) | (1<<3) | (1<<2);    //spi cs out (PB2-out10); spi data out (PB3-out11); spi clock out (PB5-out13)
  SPCR = (0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(1<<SPR1)|(1<<SPR0);  //disable SPI interrupt, enable SPI, MSB first, Master, mode 0, clock fck/128

  //init timer (8MHz generator) -  8MHz output from arduino not used because it breaks i2c transmission
/*  DDRD |= (1<<6); //8MHz clock output (PD6-out6)
  OCR0A = 0x00; //compare number 1 - this causes that timer divides 16MHz clock by 2
  TCCR0B = (1<<WGM02) | (1<<CS00); //fast PWM mode, no prescaling
  TCCR0A |= (1<<COM0A0) | (1<<WGM01) | (1<<WGM00); //toggle OC0A on compare, fast PWM mode
``*/

  //io init
  DDRB |= (1<<0); //external LED output (PB0-out8)
  DDRB |= (1<<1); //external LED output (PB1-out9)
  DDRD &= ~(1<<5) & ~(1<<7);  //switch inputs
  PORTD |= (1<<5) | (1<<7);  //switch inputs pullup
  DDRD |= (1<<2) | (1<<3) | (1<<4);  //vco select output
  
//Serial.begin(9200);
}

int32_t level_set = 0x00; //value for DAC, for setting atternuators
int32_t level_in = 0x00;  //measured value at detector output
float target_level = 0.6; //target voltage at detector output
float hys = 0.1; //allowed difference between target and detected value
int32_t r0, r1, r2, r3, r4; //register values for pll
int32_t integ;  //integer divider of RF input
int32_t frac;  //fractional divider of RF input
float freq = 3500000000;  //target frequency
float fstep = 250000;  //step of tuning frequency

volatile static int32_t i;

void loop() {
  
//////////////////////////////////////////////////////////////////////////
// AGC
//////////////////////////////////////////////////////////////////////////
  PORTB |= (1<<0) | (1<<1); //2 color LED off
  if(level_in > (target_level + hys) / 5.0 * 1024)  //if AGC overdriven
    PORTB &= ~(1<<0); //2 color LED - color 1
  if(level_in < (target_level - hys) / 5.0 * 1024)  //if AGC overdriven
    PORTB &= ~(1<<1); //2 color LED - color 2    
    
  for (i=1000; i>0; --i);  // wait before ADC sampling
  level_in = analogRead(A0);
  
//  if((~PIND & (1<<5)) && level_set > 0x00) //manual control of ADC
  if(level_in > (target_level + hys) / 5.0 * 1024 && level_set > 0x00)
  {
    --level_set;
    PORTB &= ~(1<<0) & ~(1<<1); //2 color LED - color 1 and 2
  }
//  if((~PIND & (1<<7)) && level_set < 0xff) //manual control of ADC
  if(level_in < (target_level - hys) / 5.0 * 1024 && level_set < 0xff)
  {
    ++level_set;
    PORTB &= ~(1<<0) & ~(1<<1); //2 color LED - color 1 and 2
  }

 TWIStart();
 TWIWrite(0x60<<1);
 TWIWrite(0x00);
 TWIWrite(level_set);
 TWIStop();


//////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////
// PLL
//////////////////////////////////////////////////////////////////////////

  if((~PIND & (1<<5)) && freq > FVCO1_START) //manual control of frequency
  {
    freq = freq - fstep;
  }
  if((~PIND & (1<<7)) && freq < FVCO3_STOP) //manual control of frequency
  {
    freq = freq + fstep;
  }
  integ = (int32_t)(freq / FREF);
  frac = (int32_t)(((freq / FREF) - integ) * 0x2000000);
  
  if(freq < FVCO1_STOP)
  {
    PORTD |= (1<<PD2);
    PORTD &= ~(1<<PD3) & ~(1<<PD3);
  }
  else if(freq < FVCO2_STOP)
  {
    PORTD |= (1<<PD3);
    PORTD &= ~(1<<PD2) & ~(1<<PD4);
  }
  else
  {
    PORTD |= (1<<PD4);
    PORTD &= ~(1<<PD2) & ~(1<<PD3);
  }

  r0 = 0x00000000;
  r0 = 
    0x5UL << 27             |   //analog lock detect to mux output
    integ << 15             |   //integer divider of RF input
    (0xfff&(frac>>13)) << 3 |   //msb part of fractional divider of RF input
    0x0;                        //address of r0
  r1 = 0x00000000;
  r1 = 
    (0x1fff & frac) << 15   |   //lsb part of fractional divider of RF input
    0x1;                        //address of r1
  r2 = 0x00000000;
  r2 = 
    0x0UL << 28     |  //no cycle slip reduction
    0x7UL << 24     |  //charge pump setting - 2.5???
    0x1UL << 22     |  //prescaller 8/9
    0x1UL << 21     |  // R divider enabled!!!!!!
    0x0UL << 20     |  //no ref doubler
    0x1UL << 15     |  //no input reference division
    0x2;              //address of r2
  r3 = 0x00000000;
  r3 = 
    0x0UL << 14     |  //sig-del reset enable
    0x0UL << 7      |  //24 cycles to lock detect
    0x1UL << 6      |  //positive vco characteristics
    0x0UL << 5      |  //no power down
    0x0UL << 4      |  //no 3state charge pump
    0x0UL << 3      |  //no reset
    0x3;              //address of r3
  r4 = 0x00000000;
  r4 = 
    0x3UL << 23     |  //negative bleed current on
    0x0UL << 19     |  //no clock divider mode (fastlock??)
    0x000UL << 7    |  //something with fastlock, don't care ??
    0x4;              //address of r4

  PORTB &= ~(1<<2);
  SPDR = 0xff & (r4 >> 24); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r4 >> 16); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r4 >> 8); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r4 >> 0); while(!(SPSR & (1<<SPIF)));
  PORTB |= (1<<2);
  PORTB &= ~(1<<2);
  SPDR = 0xff & (r3 >> 24); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r3 >> 16); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r3 >> 8); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r3 >> 0); while(!(SPSR & (1<<SPIF)));
  PORTB |= (1<<2);
  PORTB &= ~(1<<2);
  SPDR = 0xff & (r2 >> 24); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r2 >> 16); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r2 >> 8); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r2 >> 0); while(!(SPSR & (1<<SPIF)));
  PORTB |= (1<<2);
  PORTB &= ~(1<<2);
  SPDR = 0xff & (r1 >> 24); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r1 >> 16); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r1 >> 8); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r1 >> 0); while(!(SPSR & (1<<SPIF)));
  PORTB |= (1<<2);
  PORTB &= ~(1<<2);
  SPDR = 0xff & (r0 >> 24); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r0 >> 16); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r0 >> 8); while(!(SPSR & (1<<SPIF)));
  SPDR = 0xff & (r0 >> 0); while(!(SPSR & (1<<SPIF)));
  PORTB |= (1<<2);
/*
  PORTB |= (1<<0); //turn the LED on (PB0-out8)
  for (i=10000; i>0; --i);
  PORTB &= ~(1<<0); //turn the LED off (PB0-out8)
  for (i=10000; i>0; --i);*/
//////////////////////////////////////////////////////////////////////////

}
