close all;
clear;
clc

addpath('/opt/openEMS/share/openEMS/matlab');
addpath('/opt/openEMS/share/CSXCAD/matlab');
addpath('/opt/openEMS/share/hyp2mat/matlab');
addpath('/opt/openEMS/share/CTB/matlab');


###############################
## PREPARE SIMULATION MODEL
###############################

#constants
physical_constants; #import physical_constants
impedance = 50; #reference impedance
fmin = 4.5e9;  #min frquency
fmax = 10e9;  #max frquency
er = 4.5; #FR-4 permittivity
tan_loss = 0.02;
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*er; #FR-4 losses



#structure dimensions
subs_thick = 0.001;  #pcb thickness
res_l_ext = 0.00603;  #length of the coupled resonator
res_l_int = 0.00565;  #length of other resonators
res_tap = 0.00155;  #position of tap (from ground vias)
res_w = 0.0013;  #width of the resonator
feed_l = 0.001; #length of weak feed line
feed_w = 0.0004; #width of weak feed line
feed_g = 0.0015; #feed line weak coupling gap
filt_size_x = 0.017;
filt_size_y = 0.01203;
filt_start_y = -((filt_size_y-0.002-res_l_int)/2+res_tap+0.001)
filt_stop_y = filt_start_y+filt_size_y;
res_s12 = 0.00084;  #spacing between two resonators
res_s23 = 0.0011;  #spacing between next resonators
feed1_x = -3*res_w/2-res_s23-feed_l-feed_g;
feed2_x = res_w/2+feed_l+feed_g;

#mesh dimensions
res_coarse = c0 / fmax / 120;
res_fine_x = res_coarse / 4;
res_fine_y = res_coarse / 4;
mesh_start_x = -filt_size_x/2 - 0.004;
mesh_stop_x = filt_size_x/2 + 0.004;
mesh_start_y = filt_start_y - 0.003;
mesh_stop_y = filt_stop_y + 0.003;
mesh_start_z = 0;
mesh_stop_z = subs_thick + 0.007 + 0.003;

###############
#
#  LAST RESULT
#   m23_f0 =  6462006806.52571
#   m23 =  0.14774
#
###############

#setup FDTD
FDTD = InitFDTD('EndCriteria', 1e-5);
FDTD = SetGaussExcite(FDTD, (fmax+fmin)/2, (fmax-fmin)/2);  #finite energy excitation
BC = {'PML_8' 'PML_8' 'MUR' 'MUR' 'PEC' 'MUR'}; #boundary conditions: absorber around
FDTD = SetBoundaryCond(FDTD, BC);

#setup CSX
CSX = InitCSX();

#add materials
CSX = AddMetal(CSX,'Metal');  #new metal material
CSX = AddMetal(CSX,'Metal2');  #new metal material
CSX = AddMaterial(CSX, 'FR-4'); #new substrate material
CSX = SetMaterialProperty(CSX, 'FR-4', 'Epsilon', er, 'Kappa', kap);  #setting permittivity of the substrate
#add substrate
start = [-filt_size_x/2, filt_start_y, 0];
stop = [filt_size_x/2, filt_stop_y, subs_thick];
CSX = AddBox(CSX, 'FR-4', 400, start, stop);

CSX = ImportSTL(CSX, 'Metal', 500, '/home/michal/ELEKTRONIKA/asdf_sa/LO1rev2/design/lo_filter_3bands/stl/bpf67_m23.stl');
CSX = ImportSTL(CSX, 'Metal2', 500, '/home/michal/ELEKTRONIKA/asdf_sa/LO1rev2/design/lo_filter_3bands/stl/bpf67_shield.stl');

#draw feed lines
start = [feed1_x, -feed_w/2, subs_thick];
stop = [feed1_x+feed_l, feed_w/2, subs_thick];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [feed2_x, -2*res_tap+res_l_int-feed_w/2, subs_thick];
stop = [feed2_x-feed_l, -2*res_tap+res_l_int+feed_w/2, subs_thick];
CSX = AddBox(CSX, 'Metal', 500, start, stop);


#add ports
start = [feed1_x, -feed_w/2, 0];
stop = [feed1_x, feed_w/2, subs_thick];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);
start = [feed2_x, -2*res_tap+res_l_int-feed_w/2, 0];
stop = [feed2_x, -2*res_tap+res_l_int+feed_w/2, subs_thick];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 0 1]);


#setup mesh
mesh.x = SmoothMeshLines([feed1_x, ...
      -3*res_w/2-res_s23-2/3*res_fine_x, -3*res_w/2-res_s23+1/3*res_fine_x,...
      -1*res_w/2-res_s23-1/3*res_fine_x, -1*res_w/2-res_s23+2/3*res_fine_x,...
      -res_w/2-2/3*res_fine_x, -res_w/2+1/3*res_fine_x,...
      res_w/2-1/3*res_fine_x, res_w/2+2/3*res_fine_x,...
      feed2_x], res_fine_x, 1.5);
mesh.x = SmoothMeshLines([mesh_start_x, mesh.x, mesh_stop_x], res_coarse, 1.5);
mesh.y = SmoothMeshLines([filt_start_y+0.001, ...
      -res_tap-2/3*res_fine_y, -res_tap+1/3*res_fine_y,...
      res_l_int-res_tap-1/3*res_fine_y, res_l_int-res_tap+2/3*res_fine_y,...
       filt_stop_y-0.001], res_fine_y, 1.5);
mesh.y = SmoothMeshLines([mesh_start_y, mesh.y, mesh_stop_y], res_coarse, 1.5);
mesh.z = SmoothMeshLines([0, subs_thick, subs_thick+0.002], res_fine_y, 1.5);
mesh.z = SmoothMeshLines([mesh_start_z, mesh.z, mesh_stop_z], res_coarse, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path_m23';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

#S11 plot
f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s21 = port{2}.uf.ref ./ port{1}.uf.inc;
figure(1);
hold on;
grid on;
plot(f/1e9, 20*log10(abs(s21)), 'g-', 'LineWidth', 2);
legend('S_{21}');
ylabel('magnitudes (dB)','FontSize',12);
xlabel('frequency (GHz) \rightarrow','FontSize',12);

#results processing
tmp = -1000;
for i = 1:length(f)
  if 20*log10(abs(s21(i))) > tmp
    tmp = 20*log10(abs(s21(i)));
    m23_fpeak1 = f(i);
  end;
  if ( (tmp-3 > 20*log10(abs(s21(i)))) && (20*log10(abs(s21(i))) > -60) )
    break;
  end;
end;
tmp = -1000;
for i = length(f):-1:1
  if 20*log10(abs(s21(i))) > tmp
    tmp = 20*log10(abs(s21(i)));
    m23_fpeak2 = f(i);
  end;
  if ( (tmp-3 > 20*log10(abs(s21(i)))) && (20*log10(abs(s21(i))) > -60) )
    break;
  end;
end;

m23_f0 = sqrt(m23_fpeak2*m23_fpeak1)
m23 = (m23_fpeak2 - m23_fpeak1) / m23_f0
