close all
clear
clc

addpath('/opt/openEMS/share/openEMS/matlab');
addpath('/opt/openEMS/share/CSXCAD/matlab');
addpath('/opt/openEMS/share/hyp2mat/matlab');
addpath('/opt/openEMS/share/CTB/matlab');
physical_constants; #import physical_constants

###############################
## PREPARE SIMULATION MODEL
###############################

#variables

fmin = 1e9;
fmax = 10e9;
impedance = 50;
eps_r = 4.5; #FR-4 permability
tan_loss = 0.02; #FR-4 losses (as tangens)
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*eps_r; #FR-4 losses (as kappa)
resolution = c0 / (fmax*sqrt(eps_r)) / 120;
resolution_fine = resolution / 3;

subst_thick = 1e-3;
line_l = 15e-3;
line_w = 0.4e-3;
line_s = 0.2e-3;
line_gap = 0.8e-3;
sub_w = 4e-3;
viasize = 0.3e-3;
viaring = 0.7e-3;
bond_l = 1.6e-3;
bond_w = 0.8e-3;
bond_h = 0.45e-3;
bond_t = 0.2e-3;
boxsize_x = line_l*1.3;
boxsize_y = sub_w*2;
boxsize_z = subst_thick*5;

#initiate and setup simulation structures and parameters
FDTD = InitFDTD();
FDTD = SetGaussExcite(FDTD, fmin, fmax);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PEC' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
CSX = InitCSX();

#definitions of used materials
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);
CSX = AddConductingSheet(CSX,'Copper', 56e6, 16e-6);
CSX = AddMetal(CSX,'Metal', 56e6, 16e-6);


#geometry draw:
CSX = ImportSTL(CSX, 'Metal', 500, '/home/michal/ELEKTRONIKA/asdf_sa/LO1rev2/design/doubler/4l/c.stl');

#substrate
start = [-line_l/2-1e-3, -sub_w/2, 0];
stop = [line_l/2+1e-3, sub_w/2, subst_thick];
CSX = AddBox(CSX, 'FR4', 400, start, stop);
#ground
start = [-line_l/2, -sub_w/2, 0];
stop = [line_l/2, sub_w/2, 0];
#CSX = AddBox(CSX, 'Copper', 500, start, stop);
#port conn
start = [-line_l/2-0.8e-3, 0.65e-3+0.5e-3, subst_thick];
stop = [-line_l/2-1e-3, 0.65e-3-0.5e-3, subst_thick];
CSX = AddBox(CSX, 'Copper', 500, start, stop);
#bridges
start = [line_gap/2, bond_l/2, subst_thick];
stop = [line_gap/2+bond_w, bond_l/2-bond_t, subst_thick+bond_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [line_gap/2, -bond_l/2+bond_t, subst_thick];
stop = [line_gap/2+bond_w, -bond_l/2, subst_thick+bond_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [line_gap/2, bond_l/2, subst_thick+bond_h];
stop = [line_gap/2+bond_w, -bond_l/2, subst_thick+bond_h+bond_t];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [-line_gap/2, bond_l/2, subst_thick];
stop = [-line_gap/2-bond_w, bond_l/2-bond_t, subst_thick+bond_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [-line_gap/2, -bond_l/2+bond_t, subst_thick];
stop = [-line_gap/2-bond_w, -bond_l/2, subst_thick+bond_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [-line_gap/2, bond_l/2, subst_thick+bond_h];
stop = [-line_gap/2-bond_w, -bond_l/2, subst_thick+bond_h+bond_t];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

#setup mesh
mesh.x = SmoothMeshLines([-line_l/2-0.5e-3, line_l/2], resolution_fine, 1.5);
mesh.x = SmoothMeshLines([-boxsize_x/2, -line_l/2-1e-3, mesh.x, line_l/2, boxsize_x/2], resolution, 1.5);
mesh.y = SmoothMeshLines([-3*line_w/2-2*line_s, 5*line_w/2+3*line_s+viaring/2], resolution_fine, 1.5);
mesh.y = SmoothMeshLines([-boxsize_y/2, mesh.y, boxsize_y/2], resolution, 1.5);
mesh.z = SmoothMeshLines([0, subst_thick, 2*subst_thick], resolution_fine, 1.5);
mesh.z = SmoothMeshLines([mesh.z, boxsize_z], resolution, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


#add ports
start = [-line_l/2-1e-3 0.65e-3-0.5e-3 0];
stop = [-line_l/2-1e-3 0.65e-3+0.5e-3 subst_thick];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);
start = [-line_gap/2 -line_w/2-line_s 0];
stop = [-line_gap/2 -3*line_w/2-line_s subst_thick];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 0 1]);
start = [line_gap/2 -line_w/2-line_s 0];
stop = [line_gap/2 -3*line_w/2-line_s subst_thick];
[CSX,port{3}] = AddLumpedPort(CSX, 999, 3, impedance, start, stop, [0 0 1]);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s11 = port{1}.uf.ref ./ port{1}.uf.inc;
s21 = port{2}.uf.ref ./ port{1}.uf.inc;
s31 = port{3}.uf.ref ./ port{1}.uf.inc;
figure(1);
hold on;
grid on;
plot(f/1e9, 20*log10(abs(s11)), 'k-', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s21)), 'r-', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s31)), 'r-', 'LineWidth', 2);

legend('S_{11}','S_{21}','S_{31}');
ylabel('S-Parameter (dB)','FontSize',12);
xlabel('frequency (GHz) \rightarrow','FontSize',12);

